@@ -75,12 +75,9 @@
  */
 
 struct _PanelAppletPrivate {
-	GtkWidget         *plug;
 	GtkWidget         *applet;
 	GDBusConnection   *connection;
 
-	gboolean           out_of_process;
-
 	char              *id;
 	GClosure          *closure;
 	char              *object_path;
@@ -93,11 +90,6 @@ struct _PanelAppletPrivate {
 
 	PanelAppletFlags   flags;
 	PanelAppletOrient  orient;
-	char              *background;
-	GtkWidget         *background_widget;
-
-	int                previous_width;
-	int                previous_height;
 
         int               *size_hints;
         int                size_hints_len;
@@ -109,7 +101,6 @@ struct _PanelAppletPrivate {
 
 enum {
         CHANGE_ORIENT,
-        CHANGE_BACKGROUND,
 	MOVE_FOCUS_OUT_OF_APPLET,
         LAST_SIGNAL
 };
@@ -118,19 +109,16 @@ static guint panel_applet_signals [LAST_SIGNAL];
 
 enum {
 	PROP_0,
-	PROP_OUT_OF_PROCESS,
 	PROP_ID,
 	PROP_CLOSURE,
 	PROP_CONNECTION,
 	PROP_SETTINGS_PATH,
 	PROP_ORIENT,
-	PROP_BACKGROUND,
 	PROP_FLAGS,
 	PROP_SIZE_HINTS,
 	PROP_LOCKED_DOWN
 };
 
-static void       panel_applet_handle_background   (PanelApplet       *applet);
 static GAction   *panel_applet_menu_get_action     (PanelApplet       *applet,
 						    const gchar       *action);
 static void       panel_applet_menu_update_actions (PanelApplet       *applet);
@@ -426,6 +414,27 @@ panel_applet_get_gtk_orientation (PanelApplet *applet)
 	return GTK_ORIENTATION_HORIZONTAL;
 }
 
+/**
+ * panel_applet_add_text_class:
+ * @applet: a #PanelApplet
+ * @widget: a #GtkWidget
+ *
+ * Use this function to add css class to widgets that are visible on panel
+ * and shows text.
+ *
+ * Since: 3.22
+ */
+void
+panel_applet_add_text_class (PanelApplet *applet,
+                             GtkWidget   *widget)
+{
+  GtkStyleContext *context;
+
+  context = gtk_widget_get_style_context (widget);
+
+  gtk_style_context_add_class (context, "gp-text-color");
+}
+
 /* Applets cannot set their orientation, so API is not public. */
 static void
 panel_applet_set_orient (PanelApplet      *applet,
@@ -871,7 +880,6 @@ panel_applet_finalize (GObject *object)
 
 	g_free (applet->priv->size_hints);
 	g_free (applet->priv->settings_path);
-	g_free (applet->priv->background);
 	g_free (applet->priv->id);
 
 	/* closure is owned by the factory */
@@ -915,7 +923,10 @@ panel_applet_position_menu (GtkMenu   *menu,
 	PanelApplet    *applet;
 	GtkAllocation   allocation;
 	GtkRequisition  requisition;
+	GdkDisplay     *display;
+	GdkSeat        *seat;
 	GdkDevice      *device;
+	GdkWindow      *window;
 	GdkScreen      *screen;
 	int             menu_x = 0;
 	int             menu_y = 0;
@@ -926,16 +937,17 @@ panel_applet_position_menu (GtkMenu   *menu,
 
 	applet = PANEL_APPLET (widget);
 
+	display = gtk_widget_get_display (widget);
+	seat = gdk_display_get_default_seat (display);
+	device = gdk_seat_get_pointer (seat);
 	screen = gtk_widget_get_screen (widget);
-	
-	gtk_menu_set_screen(menu, screen);
+	window = gtk_widget_get_window (widget);
+
+	gtk_menu_set_screen (menu, screen);
 
 	gtk_widget_get_preferred_size (GTK_WIDGET (menu), &requisition, NULL);
-	gdk_window_get_origin (gtk_widget_get_window (widget),
-			       &menu_x, &menu_y);
-	device = gdk_device_manager_get_client_pointer (gdk_display_get_device_manager (gtk_widget_get_display (widget)));
-	gdk_window_get_device_position(gtk_widget_get_window (widget), device, &pointer_x, &pointer_y, NULL);
-
+	gdk_window_get_origin (window, &menu_x, &menu_y);
+	gdk_window_get_device_position (window, device, &pointer_x, &pointer_y, NULL);
 	gtk_widget_get_allocation (widget, &allocation);
 
 	menu_x += allocation.x;
@@ -1053,67 +1065,6 @@ panel_applet_can_focus (GtkWidget *widget)
 	return !container_has_focusable_child (GTK_CONTAINER (widget));
 }
 
-/* Taken from libbonoboui/bonobo/bonobo-plug.c */
-static gboolean
-panel_applet_button_event (PanelApplet    *applet,
-                           GdkEventButton *event)
-{
-	GtkWidget *widget;
-	GdkWindow *window;
-	GdkWindow *socket_window;
-	XEvent     xevent;
-
-	if (!applet->priv->out_of_process)
-		return FALSE;
-
-	widget = applet->priv->plug;
-
-	if (!gtk_widget_is_toplevel (widget))
-		return FALSE;
-
-	window = gtk_widget_get_window (widget);
-	socket_window = gtk_plug_get_socket_window (GTK_PLUG (widget));
-
-	if (event->type == GDK_BUTTON_PRESS) {
-		xevent.xbutton.type = ButtonPress;
-
-		/* X does an automatic pointer grab on button press
-		 * if we have both button press and release events
-		 * selected.
-		 * We don't want to hog the pointer on our parent.
-		 */
-		gdk_device_ungrab (event->device, GDK_CURRENT_TIME);
-	} else {
-		xevent.xbutton.type = ButtonRelease;
-	}
-
-	xevent.xbutton.display     = GDK_WINDOW_XDISPLAY (window);
-	xevent.xbutton.window      = GDK_WINDOW_XID (socket_window);
-	xevent.xbutton.root        = GDK_WINDOW_XID (gdk_screen_get_root_window
-							 (gdk_window_get_screen (window)));
-	/*
-	 * FIXME: the following might cause
-	 *        big problems for non-GTK apps
-	 */
-	xevent.xbutton.x           = 0;
-	xevent.xbutton.y           = 0;
-	xevent.xbutton.x_root      = 0;
-	xevent.xbutton.y_root      = 0;
-	xevent.xbutton.state       = event->state;
-	xevent.xbutton.button      = event->button;
-	xevent.xbutton.same_screen = TRUE; /* FIXME ? */
-
-	gdk_error_trap_push ();
-
-	XSendEvent (GDK_WINDOW_XDISPLAY (window),
-		    GDK_WINDOW_XID (socket_window),
-		    False, NoEventMask, &xevent);
-
-	gdk_error_trap_pop_ignored ();
-
-	return TRUE;
-}
-
 static gboolean
 panel_applet_button_press (GtkWidget      *widget,
 			   GdkEventButton *event)
@@ -1140,16 +1091,7 @@ panel_applet_button_press (GtkWidget      *widget,
 		return TRUE;
 	}
 
-	return panel_applet_button_event (applet, event);
-}
-
-static gboolean
-panel_applet_button_release (GtkWidget      *widget,
-			     GdkEventButton *event)
-{
-	PanelApplet *applet = PANEL_APPLET (widget);
-
-	return panel_applet_button_event (applet, event);
+	return FALSE;
 }
 
 static void
@@ -1198,54 +1140,6 @@ panel_applet_get_request_mode (GtkWidget *widget)
         return GTK_SIZE_REQUEST_WIDTH_FOR_HEIGHT;
 }
 
-static void
-panel_applet_get_preferred_width (GtkWidget *widget,
-                                  int       *minimum_width,
-                                  int       *natural_width)
-{
-        int focus_width = 0;
-
-        GTK_WIDGET_CLASS (panel_applet_parent_class)->get_preferred_width (widget,
-                                                                           minimum_width,
-                                                                           natural_width);
-        if (!panel_applet_can_focus (widget))
-                return;
-
-        /* We are deliberately ignoring focus-padding here to
-         * save valuable panel real estate.
-         */
-        gtk_widget_style_get (widget,
-                              "focus-line-width", &focus_width,
-                              NULL);
-
-        *minimum_width += 2 * focus_width;
-        *natural_width += 2 * focus_width;
-}
-
-static void
-panel_applet_get_preferred_height (GtkWidget *widget,
-                                  int       *minimum_height,
-                                  int       *natural_height)
-{
-        int focus_width = 0;
-
-        GTK_WIDGET_CLASS (panel_applet_parent_class)->get_preferred_height (widget,
-                                                                            minimum_height,
-                                                                            natural_height);
-        if (!panel_applet_can_focus (widget))
-                return;
-
-        /* We are deliberately ignoring focus-padding here to
-         * save valuable panel real estate.
-         */
-        gtk_widget_style_get (widget,
-                              "focus-line-width", &focus_width,
-                              NULL);
-
-        *minimum_height += 2 * focus_width;
-        *natural_height += 2 * focus_width;
-}
-
 static void
 panel_applet_size_allocate (GtkWidget     *widget,
 			    GtkAllocation *allocation)
@@ -1254,27 +1148,17 @@ panel_applet_size_allocate (GtkWidget     *widget,
 	GtkBin        *bin;
 	GtkWidget     *child;
 	int            border_width;
-	int            focus_width = 0;
-	PanelApplet   *applet;
 
 	if (!panel_applet_can_focus (widget)) {
 		GTK_WIDGET_CLASS (panel_applet_parent_class)->size_allocate (widget, allocation);
 	} else {
-		/*
-		 * We are deliberately ignoring focus-padding here to
-		 * save valuable panel real estate.
-		 */
-		gtk_widget_style_get (widget,
-				      "focus-line-width", &focus_width,
-				      NULL);
-
 		border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
 
 		gtk_widget_set_allocation (widget, allocation);
 		bin = GTK_BIN (widget);
 
-		child_allocation.x = focus_width;
-		child_allocation.y = focus_width;
+		child_allocation.x = 0;
+		child_allocation.y = 0;
 
 		child_allocation.width  = MAX (allocation->width  - border_width * 2, 0);
 		child_allocation.height = MAX (allocation->height - border_width * 2, 0);
@@ -1286,23 +1170,10 @@ panel_applet_size_allocate (GtkWidget     *widget,
 						child_allocation.width,
 						child_allocation.height);
 
-		child_allocation.width  = MAX (child_allocation.width  - 2 * focus_width, 0);
-		child_allocation.height = MAX (child_allocation.height - 2 * focus_width, 0);
-
 		child = gtk_bin_get_child (bin);
 		if (child)
 			gtk_widget_size_allocate (child, &child_allocation);
 	}
-
-	applet = PANEL_APPLET (widget);
-
-	if (applet->priv->previous_height != allocation->height ||
-	    applet->priv->previous_width  != allocation->width) {
-		applet->priv->previous_height = allocation->height;
-		applet->priv->previous_width = allocation->width;
-
-		panel_applet_handle_background (applet);
-	}
 }
 
 static gboolean
@@ -1311,7 +1182,6 @@ panel_applet_draw (GtkWidget *widget,
 {
         GtkStyleContext *context;
 	int border_width;
-	int focus_width = 0;
 	gdouble x, y, width, height;
 
 	GTK_WIDGET_CLASS (panel_applet_parent_class)->draw (widget, cr);
@@ -1322,14 +1192,6 @@ panel_applet_draw (GtkWidget *widget,
         width = gtk_widget_get_allocated_width (widget);
         height = gtk_widget_get_allocated_height (widget);
 
-	/*
-	 * We are deliberately ignoring focus-padding here to
-	 * save valuable panel real estate.
-	 */
-	gtk_widget_style_get (widget,
-			      "focus-line-width", &focus_width,
-			      NULL);
-
 	border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
 
 	x = 0; // FIXME: border_width ?
@@ -1397,216 +1259,6 @@ panel_applet_focus (GtkWidget        *widget,
 	return ret;
 }
 
-static cairo_surface_t *
-panel_applet_create_foreign_surface_for_display (GdkDisplay *display,
-                                                 GdkVisual  *visual,
-                                                 Window      xid)
-{
-        Window window;
-        gint x, y;
-        guint width, height, border, depth;
-
-        if (!XGetGeometry (GDK_DISPLAY_XDISPLAY (display), xid, &window,
-                           &x, &y, &width, &height, &border, &depth))
-                return NULL;
-
-        return cairo_xlib_surface_create (GDK_DISPLAY_XDISPLAY (display),
-                                          xid, gdk_x11_visual_get_xvisual (visual),
-                                          width, height);
-}
-
-static cairo_pattern_t *
-panel_applet_get_pattern_from_pixmap (PanelApplet *applet,
-                                      Window       xid,
-                                      int          x,
-                                      int          y)
-{
-	GdkWindow       *window;
-	int              width;
-	int              height;
-        cairo_t         *cr;
-        cairo_surface_t *background;
-        cairo_surface_t *surface;
-        cairo_pattern_t *pattern;
-
-	g_return_val_if_fail (PANEL_IS_APPLET (applet), NULL);
-
-	if (!gtk_widget_get_realized (GTK_WIDGET (applet)))
-		return NULL;
-
-        window = gtk_widget_get_window (GTK_WIDGET (applet));
-
-        gdk_error_trap_push ();
-        background = panel_applet_create_foreign_surface_for_display (gdk_window_get_display (window),
-                                                                      gdk_window_get_visual (window),
-                                                                      xid);
-        gdk_error_trap_pop_ignored ();
-
-        /* background can be NULL if the user changes the background very fast.
-         * We'll get the next update, so it's not a big deal. */
-        if (!background ||
-            cairo_surface_status (background) != CAIRO_STATUS_SUCCESS) {
-                if (background)
-                        cairo_surface_destroy (background);
-                return NULL;
-        }
-
-        width = gdk_window_get_width (window);
-        height = gdk_window_get_height (window);
-        surface = cairo_image_surface_create (CAIRO_FORMAT_RGB24, width, height);
-
-        gdk_error_trap_push ();
-        cr = cairo_create (surface);
-        cairo_set_source_surface (cr, background, -x, -y);
-        cairo_rectangle (cr, 0, 0, width, height);
-        cairo_fill (cr);
-        gdk_error_trap_pop_ignored ();
-
-        cairo_surface_destroy (background);
-
-	pattern = NULL;
-
-	if (cairo_status (cr) == CAIRO_STATUS_SUCCESS)
-		pattern = cairo_pattern_create_for_surface (surface);
-
-        cairo_destroy (cr);
-        cairo_surface_destroy (surface);
-
-	return pattern;
-}
-
-/**
- * panel_applet_get_background:
- * @applet: a #PanelApplet.
- *
- * Gets the background pattern for @applet, or %NULL if there is none.
- *
- * Returns: (transfer full): a new #cairo_pattern_t to use as background for
- * @applet.
- **/
-cairo_pattern_t *
-panel_applet_get_background (PanelApplet *applet)
-{
-        cairo_pattern_t *pattern = NULL;
-        GVariant        *variant;
-        GVariantIter     iter;
-        GError          *error = NULL;
-
-        g_return_val_if_fail (PANEL_IS_APPLET (applet), NULL);
-
-	if (!gtk_widget_get_realized (GTK_WIDGET (applet)) || !applet->priv->background)
-		return NULL;
-
-        variant = g_variant_parse (NULL, applet->priv->background,
-                                   NULL, NULL, &error);
-        if (!variant) {
-                g_warning ("Error parsing background %s: %s\n", applet->priv->background, error->message);
-                g_error_free (error);
-                return NULL;
-        }
-
-        g_variant_iter_init (&iter, variant);
-        switch (g_variant_iter_n_children (&iter)) {
-        case 4: {
-                gdouble red, green, blue, alpha;
-
-                g_variant_get (variant, "(dddd)", &red, &green, &blue, &alpha);
-                pattern = cairo_pattern_create_rgba (red, green, blue, alpha);
-        }
-                break;
-        case 3: {
-                guint32 xid;
-                int x, y;
-
-                g_variant_get (variant, "(uii)", &xid, &x, &y);
-                pattern = panel_applet_get_pattern_from_pixmap (applet, xid, x, y);
-                if (!pattern)
-                        g_warning ("Failed to get pixmap %d, %d, %d", xid, x, y);
-        }
-                break;
-        default:
-                break;
-        }
-
-        g_variant_unref (variant);
-
-        return pattern;
-}
-
-static void
-panel_applet_set_background_string (PanelApplet *applet,
-				    const gchar *background)
-{
-	if (applet->priv->background == background)
-		return;
-
-	if (g_strcmp0 (applet->priv->background, background) == 0)
-		return;
-
-	if (applet->priv->background)
-		g_free (applet->priv->background);
-	applet->priv->background = background ? g_strdup (background) : NULL;
-	panel_applet_handle_background (applet);
-
-	g_object_notify (G_OBJECT (applet), "background");
-}
-
-static void
-panel_applet_style_updated (GtkWidget *widget)
-{
-  PanelApplet *applet;
-
-  applet = PANEL_APPLET (widget);
-
-  GTK_WIDGET_CLASS (panel_applet_parent_class)->style_updated (widget);
-
-  panel_applet_handle_background (applet);
-}
-
-static void
-panel_applet_update_background_for_widget (GtkWidget       *widget,
-					   cairo_pattern_t *pattern)
-{
-	GdkWindow *window;
-
-	window = gtk_widget_get_window (widget);
-
-	if (!window)
-		return;
-
-	gdk_window_set_background_pattern (window,
-	                                   pattern);
-}
-
-static void
-panel_applet_handle_background (PanelApplet *applet)
-{
-        cairo_pattern_t *pattern;
-
-	pattern = panel_applet_get_background (applet);
-
-	if (applet->priv->background_widget)
-		panel_applet_update_background_for_widget (applet->priv->background_widget,
-							   pattern);
-	else
-		panel_applet_update_background_for_widget (GTK_WIDGET (applet), pattern);
-
-        g_signal_emit (G_OBJECT (applet),
-                        panel_applet_signals [CHANGE_BACKGROUND],
-                        0, pattern);
-        if (pattern)
-		cairo_pattern_destroy (pattern);
-}
-
-static void
-panel_applet_realize (GtkWidget *widget)
-{
-	GTK_WIDGET_CLASS (panel_applet_parent_class)->realize (widget);
-
-	if (PANEL_APPLET (widget)->priv->background)
-		panel_applet_handle_background (PANEL_APPLET (widget));
-}
-
 static void
 panel_applet_move_focus_out_of_applet (PanelApplet      *applet,
 				       GtkDirectionType  dir)
@@ -1630,9 +1282,6 @@ panel_applet_get_property (GObject    *object,
 	PanelApplet *applet = PANEL_APPLET (object);
 
 	switch (prop_id) {
-	case PROP_OUT_OF_PROCESS:
-		g_value_set_boolean (value, applet->priv->out_of_process);
-		break;
 	case PROP_ID:
 		g_value_set_string (value, applet->priv->id);
 		break;
@@ -1648,9 +1297,6 @@ panel_applet_get_property (GObject    *object,
 	case PROP_ORIENT:
 		g_value_set_uint (value, applet->priv->orient);
 		break;
-	case PROP_BACKGROUND:
-		g_value_set_string (value, applet->priv->background);
-		break;
 	case PROP_FLAGS:
 		g_value_set_uint (value, applet->priv->flags);
 		break;
@@ -1685,9 +1331,6 @@ panel_applet_set_property (GObject      *object,
 	PanelApplet *applet = PANEL_APPLET (object);
 
 	switch (prop_id) {
-	case PROP_OUT_OF_PROCESS:
-		applet->priv->out_of_process = g_value_get_boolean (value);
-		break;
 	case PROP_ID:
 		applet->priv->id = g_value_dup_string (value);
 		break;
@@ -1709,9 +1352,6 @@ panel_applet_set_property (GObject      *object,
 	case PROP_ORIENT:
 		panel_applet_set_orient (applet, g_value_get_uint (value));
 		break;
-	case PROP_BACKGROUND:
-		panel_applet_set_background_string (applet, g_value_get_string (value));
-		break;
 	case PROP_FLAGS:
 		panel_applet_set_flags (applet, g_value_get_uint (value));
 		break;
@@ -1750,7 +1390,7 @@ panel_applet_setup (PanelApplet *applet)
 {
 	GValue   value = {0, };
 	GArray  *params;
-	gint     i;
+	guint    i;
 	gboolean ret;
 
 	g_assert (applet->priv->id != NULL &&
@@ -1793,6 +1433,7 @@ static void
 panel_applet_init (PanelApplet *applet)
 {
 	GtkStyleContext *context;
+	gchar *xml;
 
 	applet->priv = panel_applet_get_instance_private (applet);
 
@@ -1807,7 +1448,7 @@ panel_applet_init (PanelApplet *applet)
 
 	applet->priv->builder = gtk_builder_new ();
 
-	gchar *xml = g_strdup_printf (panel_menu_ui, _("_Remove From Panel"), _("_Move"));
+	xml = g_strdup_printf (panel_menu_ui, _("_Remove From Panel"), _("_Move"));
 	gtk_builder_add_from_string (applet->priv->builder, xml, -1, NULL);
 	g_free (xml);
 
@@ -1824,40 +1465,13 @@ panel_applet_init (PanelApplet *applet)
 	panel_applet_composited_changed (GTK_WIDGET (applet));
 }
 
-static GObject *
-panel_applet_constructor (GType                  type,
-                          guint                  n_construct_properties,
-                          GObjectConstructParam *construct_properties)
-{
-	GObject     *object;
-	PanelApplet *applet;
-
-	object = G_OBJECT_CLASS (panel_applet_parent_class)->constructor (type,
-	                                                                  n_construct_properties,
-	                                                                  construct_properties);
-	applet = PANEL_APPLET (object);
-
-	if (!applet->priv->out_of_process)
-		return object;
-
-	applet->priv->plug = gtk_plug_new (0);
-
-	g_signal_connect_swapped (G_OBJECT (applet->priv->plug), "embedded",
-		                      G_CALLBACK (panel_applet_setup),
-		                      applet);
-
-	gtk_container_add (GTK_CONTAINER (applet->priv->plug), GTK_WIDGET (applet));
-
-	return object;
-}
-
 static void
 panel_applet_constructed (GObject *object)
 {
 	PanelApplet *applet = PANEL_APPLET (object);
 
 	if (!applet->priv->connection || !applet->priv->closure || !applet->priv->id) {
-		g_printerr ("Bad use of PanelApplet API: you should not create a PanelApplet object yourself. Please use panel_applet_factory_main() instead.\n");
+		g_printerr ("Bad use of PanelApplet API: you should not create a PanelApplet object yourself.\n");
 		g_assert_not_reached ();
 	}
 
@@ -1873,38 +1487,19 @@ panel_applet_class_init (PanelAppletClass *klass)
 
 	gobject_class->get_property = panel_applet_get_property;
 	gobject_class->set_property = panel_applet_set_property;
-	gobject_class->constructor = panel_applet_constructor;
 	gobject_class->constructed = panel_applet_constructed;
         gobject_class->finalize = panel_applet_finalize;
 
 	klass->move_focus_out_of_applet = panel_applet_move_focus_out_of_applet;
 
 	widget_class->button_press_event = panel_applet_button_press;
-	widget_class->button_release_event = panel_applet_button_release;
 	widget_class->composited_changed = panel_applet_composited_changed;
 	widget_class->key_press_event = panel_applet_key_press_event;
 	widget_class->get_request_mode = panel_applet_get_request_mode;
-        widget_class->get_preferred_width = panel_applet_get_preferred_width;
-        widget_class->get_preferred_height = panel_applet_get_preferred_height;
 	widget_class->size_allocate = panel_applet_size_allocate;
 	widget_class->draw = panel_applet_draw;
-	widget_class->style_updated = panel_applet_style_updated;
 	widget_class->focus = panel_applet_focus;
-	widget_class->realize = panel_applet_realize;
 
-	/**
-	 * PanelApplet:out-of-process: (skip)
-	 *
-	 * Implementation detail.
-	 **/
-	g_object_class_install_property (gobject_class,
-	                                 PROP_OUT_OF_PROCESS,
-	                                 g_param_spec_boolean ("out-of-process",
-	                                                       "out-of-process",
-	                                                       "out-of-process",
-	                                                       TRUE,
-	                                                       G_PARAM_CONSTRUCT_ONLY |
-	                                                       G_PARAM_READWRITE));
 	/**
 	 * PanelApplet:id: (skip)
 	 *
@@ -1974,18 +1569,6 @@ panel_applet_class_init (PanelAppletClass *klass)
 							    PANEL_APPLET_ORIENT_LAST,
 							    PANEL_APPLET_ORIENT_UP,
 							    G_PARAM_READWRITE));
-	/**
-	 * PanelApplet:background: (skip)
-	 *
-	 * Implementation detail.
-	 **/
-	g_object_class_install_property (gobject_class,
-					 PROP_BACKGROUND,
-					 g_param_spec_string ("background",
-							      "Background",
-							      "Panel Applet Background",
-							      NULL,
-							      G_PARAM_READWRITE));
 	/**
 	 * PanelApplet:flags:
 	 *
@@ -2044,25 +1627,6 @@ panel_applet_class_init (PanelAppletClass *klass)
 			      1,
 			      G_TYPE_UINT);
 
-        /**
-         * PanelApplet::change-background:
-         * @applet: the #PanelApplet which emitted the signal.
-         * @pattern: the new background pattern for @applet, or %NULL if there is none.
-         *
-         * Emitted when the background of @applet has changed.
-         **/
-	panel_applet_signals [CHANGE_BACKGROUND] =
-                g_signal_new ("change_background",
-                              G_TYPE_FROM_CLASS (klass),
-                              G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (PanelAppletClass, change_background),
-                              NULL,
-			      NULL,
-                              g_cclosure_marshal_VOID__BOXED,
-                              G_TYPE_NONE,
-			      1,
-			      CAIRO_GOBJECT_TYPE_PATTERN);
-
         /**
          * PanelApplet::move-focus-out-of-applet: (skip)
          * @applet: the #PanelApplet which emitted the signal.
@@ -2088,6 +1652,8 @@ panel_applet_class_init (PanelAppletClass *klass)
 	add_tab_bindings (binding_set, GDK_SHIFT_MASK, GTK_DIR_TAB_BACKWARD);
 	add_tab_bindings (binding_set, GDK_CONTROL_MASK, GTK_DIR_TAB_FORWARD);
 	add_tab_bindings (binding_set, GDK_CONTROL_MASK | GDK_SHIFT_MASK, GTK_DIR_TAB_BACKWARD);
+
+	gtk_widget_class_set_css_name (widget_class, "panel-applet");
 }
 
 static void
@@ -2138,9 +1704,6 @@ get_property_cb (GDBusConnection *connection,
 					       applet->priv->settings_path : "");
 	} else if (g_strcmp0 (property_name, "Orient") == 0) {
 		retval = g_variant_new_uint32 (applet->priv->orient);
-	} else if (g_strcmp0 (property_name, "Background") == 0) {
-		retval = g_variant_new_string (applet->priv->background ?
-					       applet->priv->background : "");
 	} else if (g_strcmp0 (property_name, "Flags") == 0) {
 		retval = g_variant_new_uint32 (applet->priv->flags);
 	} else if (g_strcmp0 (property_name, "SizeHints") == 0) {
@@ -2176,8 +1739,6 @@ set_property_cb (GDBusConnection *connection,
 		panel_applet_set_settings_path (applet, g_variant_get_string (value, NULL));
 	} else if (g_strcmp0 (property_name, "Orient") == 0) {
 		panel_applet_set_orient (applet, g_variant_get_uint32 (value));
-	} else if (g_strcmp0 (property_name, "Background") == 0) {
-		panel_applet_set_background_string (applet, g_variant_get_string (value, NULL));
 	} else if (g_strcmp0 (property_name, "Flags") == 0) {
 		panel_applet_set_flags (applet, g_variant_get_uint32 (value));
 	} else if (g_strcmp0 (property_name, "SizeHints") == 0) {
@@ -2206,7 +1767,6 @@ static const gchar introspection_xml[] =
 	    "</method>"
 	    "<property name='SettingsPath' type='s' access='readwrite'/>"
 	    "<property name='Orient' type='u' access='readwrite' />"
-	    "<property name='Background' type='s' access='readwrite'/>"
 	    "<property name='Flags' type='u' access='readwrite'/>"
 	    "<property name='SizeHints' type='ai' access='readwrite'/>"
 	    "<property name='LockedDown' type='b' access='readwrite'/>"
@@ -2246,70 +1806,8 @@ panel_applet_register_object (PanelApplet *applet)
 	}
 }
 
-static void
-panel_applet_factory_main_finalized (gpointer data,
-				     GObject *object)
-{
-	gtk_main_quit ();
-
-	if (introspection_data) {
-		g_dbus_node_info_unref (introspection_data);
-		introspection_data = NULL;
-	}
-}
-
-static int (*_x_error_func) (Display *, XErrorEvent *);
-
-static int
-_x_error_handler (Display *display, XErrorEvent *error)
-{
-	if (!error->error_code)
-		return 0;
-
-	/* If we got a BadDrawable or a BadWindow, we ignore it for now.
-	 * FIXME: We need to somehow distinguish real errors from
-	 * X-server-induced errors. Keeping a list of windows for which we
-	 * will ignore BadDrawables would be a good idea. */
-	if (error->error_code == BadDrawable ||
-	    error->error_code == BadWindow)
-		return 0;
-
-	return _x_error_func (display, error);
-}
-
-/*
- * To do graphical embedding in the X window system, GNOME Panel
- * uses the classic foreign-window-reparenting trick. The
- * GtkPlug/GtkSocket widgets are used for this purpose. However,
- * serious robustness problems arise if the GtkSocket end of the
- * connection unexpectedly dies. The X server sends out DestroyNotify
- * events for the descendants of the GtkPlug (i.e., your embedded
- * component's windows) in effectively random order. Furthermore, if
- * you happened to be drawing on any of those windows when the
- * GtkSocket was destroyed (a common state of affairs), an X error
- * will kill your application.
- *
- * To solve this latter problem, GNOME Panel sets up its own X error
- * handler which ignores certain X errors that might have been
- * caused by such a scenario. Other X errors get passed to gdk_x_error
- * normally.
- */
-static void
-_panel_applet_setup_x_error_handler (void)
-{
-	static gboolean error_handler_setup = FALSE;
-
-	if (error_handler_setup)
-		return;
-
-	error_handler_setup = TRUE;
-
-	_x_error_func = XSetErrorHandler (_x_error_handler);
-}
-
 static int
 _panel_applet_factory_main_internal (const gchar               *factory_id,
-				     gboolean                   out_process,
 				     GType                      applet_type,
 				     PanelAppletFactoryCallback callback,
 				     gpointer                   user_data)
@@ -2321,21 +1819,11 @@ _panel_applet_factory_main_internal (const gchar               *factory_id,
 	g_return_val_if_fail (callback != NULL, 1);
 	g_assert (g_type_is_a (applet_type, PANEL_TYPE_APPLET));
 
-	if (out_process)
-		_panel_applet_setup_x_error_handler ();
-
 	closure = g_cclosure_new (G_CALLBACK (callback), user_data, NULL);
-	factory = panel_applet_factory_new (factory_id, out_process, applet_type, closure);
+	factory = panel_applet_factory_new (factory_id, applet_type, closure);
 	g_closure_unref (closure);
 
 	if (panel_applet_factory_register_service (factory)) {
-		if (out_process) {
-			g_object_weak_ref (G_OBJECT (factory),
-					   panel_applet_factory_main_finalized,
-					   NULL);
-			gtk_main ();
-		}
-
 		return 0;
 	}
 
@@ -2344,42 +1832,6 @@ _panel_applet_factory_main_internal (const gchar               *factory_id,
 	return 1;
 }
 
-/**
- * panel_applet_factory_main:
- * @factory_id: identifier of an applet factory.
- * @applet_type: GType of the applet this factory creates.
- * @callback: (scope call): callback to be called when a new applet is created.
- * @data: (closure): callback data.
- *
- * Creates the applet factory for @factory_id, so that the factory can create
- * instances of the applet types it is associated with.
- *
- * Applet instances created by the applet factory will use @applet_type as
- * GType. Unless you subclass #PanelApplet, you should use %PANEL_TYPE_APPLET
- * as @applet_type.
- *
- * On creation of the applet instances, @callback is called to setup the
- * applet. If @callback returns %FALSE, the creation of the applet instance is
- * cancelled.
- *
- * If using C, it is recommended to use #PANEL_APPLET_OUT_PROCESS_FACTORY
- * instead as it will create a main() function for you.
- *
- * It can only be used once, and is incompatible with the use of
- * %PANEL_APPLET_IN_PROCESS_FACTORY and %PANEL_APPLET_OUT_PROCESS_FACTORY.
- *
- * Returns: 0 on success, 1 if there is an error.
- **/
-int
-panel_applet_factory_main (const gchar               *factory_id,
-			   GType                      applet_type,
-			   PanelAppletFactoryCallback callback,
-			   gpointer                   data)
-{
-	return _panel_applet_factory_main_internal (factory_id, TRUE, applet_type,
-						    callback, data);
-}
-
 /**
  * panel_applet_factory_setup_in_process: (skip)
  * @factory_id: identifier of an applet factory.
@@ -2399,7 +1851,7 @@ panel_applet_factory_main (const gchar               *factory_id,
  * cancelled.
  *
  * It can only be used once, and is incompatible with the use of
- * %PANEL_APPLET_IN_PROCESS_FACTORY and %PANEL_APPLET_OUT_PROCESS_FACTORY.
+ * %PANEL_APPLET_IN_PROCESS_FACTORY.
  *
  * Returns: 0 on success, 1 if there is an error.
  **/
@@ -2409,48 +1861,10 @@ panel_applet_factory_setup_in_process (const gchar               *factory_id,
 				       PanelAppletFactoryCallback callback,
 				       gpointer                   data)
 {
-	return _panel_applet_factory_main_internal (factory_id, FALSE, applet_type,
+	return _panel_applet_factory_main_internal (factory_id, applet_type,
 						    callback, data);
 }
 
-/**
- * panel_applet_set_background_widget:
- * @applet: a #PanelApplet.
- * @widget: a #GtkWidget.
- *
- * Configure #PanelApplet to automatically draw the background of the applet on
- * @widget. It is generally enough to call this function with @applet as
- * @widget.
- **/
-void
-panel_applet_set_background_widget (PanelApplet *applet,
-				    GtkWidget   *widget)
-{
-	applet->priv->background_widget = widget;
-
-	if (widget && gtk_widget_get_realized (widget)) {
-		cairo_pattern_t *pattern;
-
-		pattern = panel_applet_get_background (applet);
-		panel_applet_update_background_for_widget (widget, pattern);
-		if (pattern)
-			cairo_pattern_destroy (pattern);
-	}
-}
-
-guint32
-panel_applet_get_xid (PanelApplet *applet,
-		      GdkScreen   *screen)
-{
-	if (applet->priv->out_of_process == FALSE)
-		return 0;
-
-	gtk_window_set_screen (GTK_WINDOW (applet->priv->plug), screen);
-	gtk_widget_show (applet->priv->plug);
-
-	return gtk_plug_get_id (GTK_PLUG (applet->priv->plug));
-}
-
 const gchar *
 panel_applet_get_object_path (PanelApplet *applet)
 {
